# Copyright 2021 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import logging

from vdk.api.plugin.hook_markers import hookimpl
from vdk.internal.builtin_plugins.config.vdk_config import LOG_CONFIG
from vdk.internal.builtin_plugins.termination_message import action
from vdk.internal.builtin_plugins.termination_message import (
    writer_configuration,
)
from vdk.internal.builtin_plugins.termination_message.writer_configuration import (
    WriterConfiguration,
)
from vdk.internal.core import errors
from vdk.internal.core.config import ConfigurationBuilder
from vdk.internal.core.context import CoreContext

log = logging.getLogger(__name__)


class TerminationMessageWriterPlugin:
    @hookimpl(tryfirst=True)
    def vdk_configure(self, config_builder: ConfigurationBuilder):
        writer_configuration.add_definitions(config_builder)

    @hookimpl
    def vdk_exit(self, context: CoreContext, exit_code: int):
        self._write_termination_message(
            errors.get_blamee_overall(),  # TODO: get this from context
            errors.get_blamee_overall_user_error(),  # TODO: get this from context
            context.configuration,
        )

    def _write_termination_message(self, error_overall, user_error, configuration):
        termination_message_writer_cfg = WriterConfiguration(configuration)

        try:
            termination_message_output_file = (
                termination_message_writer_cfg.get_output_file()
            )

            if (
                termination_message_writer_cfg.get_writer_enabled()
                and termination_message_output_file
            ):
                # Hide termination messages when running locally
                show_termination_log_messages = (
                    configuration.get_value(LOG_CONFIG).lower() != "local"
                )
                termination_action = action.WriteToFileAction(
                    termination_message_output_file, show_termination_log_messages
                )

                if show_termination_log_messages:
                    log.debug(
                        f'Writing termination message to file "{termination_message_output_file}"'
                    )
                self._execute_termination_action(
                    termination_action, error_overall, user_error
                )
        except:
            log.exception("Failed to write termination message.")

    @staticmethod
    def _execute_termination_action(
        action, error_overall, user_error, execution_skipped=False
    ):
        if execution_skipped:
            action.skipped()
        elif not error_overall:
            action.success()
        elif user_error:
            action.user_error()
        else:
            action.platform_error()



# Copyright 2021 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import logging
from abc import ABC
from abc import abstractmethod

log = logging.getLogger(__name__)


class ITerminationAction(ABC):
    """
    A base class representing an action that is performed upon data job completion, depending on the result.
    """

    @abstractmethod
    def success(self):
        """
        Called when the job execution was successful.
        """
        pass

    @abstractmethod
    def user_error(self):
        """
        Called when the job execution failed with an user error.
        """
        pass

    @abstractmethod
    def platform_error(self):
        """
        Called when the job execution failed with a platform error.
        """
        pass


class WriteToFileAction(ITerminationAction):
    """
    Writes a string message to a specified file upon completion of a data job execution.

    This is intended to provide a termination status of a data job in a Kubernetes environment
    (see https://kubernetes.io/docs/tasks/debug-application-cluster/determine-reason-pod-failure/).
    Keep in mind that there is a limit to the length of the termination message.
    """

    def __init__(self, filename, show_log_messages=True):
        self.filename = filename
        self.show_log_messages = show_log_messages

    def _append_to_file(self, message):
        if not self.filename:
            return

        try:
            with open(self.filename, "a") as file:
                file.write(message)
        except OSError as e:
            if self.show_log_messages:
                log.debug(
                    f'Unable to write termination message to file "{self.filename}". {e}'
                )

    def success(self):
        self._append_to_file("Success\n")

    def user_error(self):
        self._append_to_file("User error\n")

    def platform_error(self):
        self._append_to_file("Platform error\n")

    def skipped(self):
        self._append_to_file("Skipped\n")




RELEASE_VERSION='0.0.383715870'
BUILD_DATE='Wed Oct  6 16:17:00 UTC 2021'
BUILD_MACHINE_INFO='Linux runner-mvj-ljo-project-28359933-concurrent-0rdzv5 5.4.129-62.227.amzn2.x86_64 #1 SMP Wed Jul 7 00:08:43 UTC 2021 x86_64 GNU/Linux'
GITLAB_CI_JOB_ID='1655524253'
GIT_COMMIT_SHA='2e397c9ad15360a2ea1970d543977480aaefeb00'
GIT_BRANCH='main'
